# Macro LED effect ID's
MACRO_LED_STATIC = 0x00
MACRO_LED_BLINK = 0x01

# Wave Directions
WAVE_RIGHT = 0x01
WAVE_LEFT = 0x02

# Reactive
REACTIVE_500MS = 0x01
REACTIVE_1000MS = 0x02
REACTIVE_1500MS = 0x03
REACTIVE_2000MS = 0x04

# Starlight
STARLIGHT_FAST = 0x01
STARLIGHT_NORMAL = 0x02
STARLIGHT_SLOW = 0x03

# Ripple
RIPPLE_REFRESH_RATE = 0.05

POLL_1000HZ = 1000
POLL_500HZ = 500
POLL_125HZ = 125
